 /*
 ////////////////////////////////////////////////////////////////////////////////
 * Arduino code to display scrolling characters on a 8x56 LED matrix display    *
 * Works with 7x60 display. Font is 5x7 so top row not used on 8x8 modules      *
 * Display driver code based on original by Soumit Das,B.Tech AEIE,RCCIIT Kol   *
 * Modified to use SPI and tweaked for my particular display by Adrian Smith    *
 * 74HC595 used for rows and MIC5891 used for columns. Logic inverted for rows  * 
 * Additional serial interface code by Adrian Smith + further modifications     *
 ///////////////////////////////////////////////////////////////////////////////
*/

#include <SPI.h>
#include <EEPROM.h>
#include "EEPROMAnything.h"
#include <avr/pgmspace.h> 

#define max_char 100

char msg[] ="No msg stored, send one via serial menu ";//Change the text here.
char message[max_char];
int scrollspeed=10;//Set the scroll speed ( lower=faster, 0-20

char r_char;               // reads each character
byte index = 0;            // defines the position into your array
int i;
int serial_data = 0;
int eeprom_blank = 0;
int a = 0;
int val;

int x;
int y;

int ssPin = 10;

//Rows
int clockPin = 2; //Arduino pin connected to Clock Pin 11 of 74HC595
int latchPin = 7; //Arduino pin connected to Latch Pin 12 of 74HC595
int dataPin = 3;  //Arduino pin connected to Data Pin 14 of 74HC595

//BITMAP
//Bits in this array represents one LED of the matrix
// 8 is # of rows, 6 is # of LED matrices
byte bitmap[8][9]; 

int numZones = sizeof(bitmap) / 8; // One Zone refers to one 8 x 8 Matrix ( Group of 8 columns)
int maxZoneIndex = numZones-1;
int numCols = numZones * 8;

//FONT DEFENITION

// standard ascii 5x7 font
// defines ascii characters 0x20-0x7F (32-127)
byte alphabets[][5] = {
 {0,0,0,0,0},                       // spatie      ASCII 32
 {0,0,253,0,0},                     // uitroepteken  ASCII 33
 {0,96,0,96,0},                      // "           ASCII 34
 {20,127,20,127,20},                // #           ASCII 35
 {36,42,127,42,18},                    // $           ASCII 36
 {17,2,4,8,17},                        // %           ASCII 37
 {54,73,85,34,5},                      // &           ASCII 38
 {0,0,104,112,0},                      // '           ASCII 39
 {28,34,65},                           // (           ASCII 40
 {65,34,28},                           // )           ASCII 41
 {20,8,62,8,20},                       // *           ASCII 42
 {8,8,62,8,8},                         // +           ASCII 43
 {0,0,5,6,0},                          // ,           ASCII 44
 {8,8,8,8,8},                          // -           ASCII 45
 {0,0,1,0,0},                          // .           ASCII 46
 {1,2,4,8,16},                         // /           ASCII 47
 {62,69,73,81,62},                     // 0           ASCII 48
 {0,33,127,1,0},                       // 1           ASCII 49
 {33,67,69,73,49},                     // 2           ASCII 50
 {66,65,81,105,70},                    // 3           ASCII 51
 {12,20,36,127,4},                     // 4           ASCII 52
 {113,81,81,81,78},                    // 5           ASCII 53
 {30,41,73,73,6},                      // 6           ASCII 54
 {64,64,79,80,96},                     // 7           ASCII 55
 {54,73,73,73,54},                     // 8           ASCII 56
 {48,73,73,74,60},                     // 9           ASCII 57 
 {0,0,54,54,0},                        // :           ASCII 58
 {0,0,53,54,0},                        // ;           ASCII 59
 {0,8,20,34,65},                       // <           ASCII 60
 {20,20,20,20,20},                     // =           ASCII 61
 {0,65,34,20,8},                       // >           ASCII 62
 {32,64,69,72,48},                     // ?           ASCII 63
 {38,73,77,65,62},                     // @           ASCII 64
 {31,36,68,36,31},                     // A           ASCII 65
 {127,73,73,73,54},                    // B           ASCII 66
 {62,65,65,65,34},                     // C           ASCII 67
 {127,65,65,34,28},                    // D           ASCII 68
 {127,73,73,65,65},                    // E           ASCII 69
 {127,72,72,72,64},                    // F           ASCII 70
 {62,65,65,69,38},                     // G           ASCII 71
 {127,8,8,8,127},                      // H           ASCII 72
 {0,65,127,65,0},                      // I           ASCII 73
 {2,1,1,1,126},                        // J           ASCII 74
 {127,8,20,34,65},                     // K           ASCII 75
 {127,1,1,1,1},                        // L           ASCII 76
 {127,32,16,32,127},                   // M           ASCII 77
 {127,32,16,8,127},                    // N           ASCII 78
 {62,65,65,65,62},                     // O           ASCII 79
 {127,72,72,72,48},                    // P           ASCII 80
 {62,65,69,66,61},                     // Q           ASCII 81
 {127,72,76,74,49},                    // R           ASCII 82
 {50,73,73,73,38},                     // S           ASCII 83
 {64,64,127,64,64},                    // T           ASCII 84
 {126,1,1,1,126},                      // U           ASCII 85
 {124,2,1,2,124},                      // V           ASCII 86
 {126,1,6,1,126},                      // W           ASCII 87
 {99,20,8,20,99},                      // X           ASCII 88
 {96,16,15,16,96},                     // Y           ASCII 89
 {67,69,73,81,97},                     // Z           ASCII 90
 {0,127,65,65,0},                      // [           ASCII 91
 {0,0,0,0,0},                          // \           ASCII 92
 {0,65,65,127,0},                      // ]           ASCII 93
 {16,32,64,32,16},                     // ^           ASCII 94
 {1,1,1,1,1},                          // _           ASCII 95
 {0,64,32,16,0},                       // `           ASCII 96
 {2,21,21,21,15},                      // a           ASCII 97
 {127,5,9,9,6},                        // b           ASCII 98
 {14,17,17,17,2},                      // c           ASCII 99
 {6,9,9,5,127},                        // d           ASCII 100
 {14,21,21,21,12},                     // e           ASCII 101
 {8,63,72,64,32},                      // f           ASCII 102
 {24,37,37,37,62},                     // g           ASCII 103
 {127,8,16,16,15},                     // h           ASCII 104
 {0,0,47,0,0},                         // i           ASCII 105
 {2,1,17,94,0},                        // j           ASCII 106
 {127,4,10,17,0},                      // k           ASCII 107
 {0,65,127,1,0},                       // l           ASCII 108
 {31,16,12,16,31},                     // m           ASCII 109
 {31,8,16,16,15},                      // n           ASCII 110
 {14,17,17,17,14},                     // o           ASCII 111
 {31,20,20,20,8},                      // p           ASCII 112
 {8,20,20,12,31},                      // q           ASCII 113
 {31,8,16,16,8},                       // r           ASCII 114
 {9,21,21,21,2},                       // s           ASCII 115
 {16,126,17,1,2},                      // t           ASCII 116
 {30,1,1,2,31},                        // u           ASCII 117
 {28,2,1,2,28},                        // v           ASCII 118
 {30,1,6,1,30},                        // w           ASCII 119
 {17,10,4,10,17},                      // x           ASCII 120
 {24,5,5,5,30},                        // y           ASCII 121
 {17,19,21,25,17},                     // z           ASCII 122
 {0,0,8,54,65},                        // {           ASCII 123
 {0,0,127,0,0},                        // |           ASCII 124
 {65,54,8,0,0},                        // }           ASCII 125/Z
};

void setup() 
{

 SPI.begin();
 Serial.begin(9600);
 pinMode(latchPin, OUTPUT);
 pinMode(clockPin, OUTPUT);
 pinMode(dataPin, OUTPUT);
 pinMode (ssPin,OUTPUT);
 
 //Clear bitmap
 for (int row = 0; row < 8; row++) 
 {
   for (int zone = 0; zone <= maxZoneIndex; zone++) 
   {
     bitmap[row][zone] = 0; 
   }
 }

 Serial.print(F("Scrolling message display. Up to 100 characters can be displayed"));  // F specifies do not copy text string to SRAM
 Serial.println("");
 Serial.print(F("Type message to display and press enter / CR "));
 Serial.println("");
 Serial.print(F("Do not close serial window till message has been transferred "));
 Serial.println("");
 Serial.print(F("This may take up to 30 seconds"));
 Serial.println("");

 val = EEPROM.read(a); // check if eeprom is blank e.g first time program is run on new chip or eeprom has been erased
  
  if (val == 255)
   {
    eeprom_blank = 1;
   }
  
 if (eeprom_blank == 0)
 {
  EEPROM_readAnything(0, message);
  memset(&msg[0], 0, sizeof(msg)); // erase msg array as we have message stored in eeprom
  strcpy(msg,message); // copy message from eeprom to msg array
 }

  // no else as if the eeprom is blank display hard coded message defined earlier. Reset required after uploading for 1st time.

}

//FUNCTIONS
// Displays bitmap array in the matrix
void RefreshDisplay()
{
 for (int row = 0; row < 8; row++) 
 {
   int rowbit = 1 << row;
   PORTD &= ~(1<<PD7); // set latch pin low
   shiftOut(dataPin, clockPin, MSBFIRST, rowbit ^ 0xFF);   //Transmit data & invert bits
   PORTD |= (1<<PD7); // set latch pin high
   
   //Start sending column bytes
   SPI.beginTransaction(SPISettings(4000000, MSBFIRST, SPI_MODE0)); //2000000 MIC5891 max speed is 3.3Mhz but seems fine up to 5Mhz
   PORTB |= (1<<PB2); // set pin 10 (SS) high
  
   //Shift out to each matrix
   for (int zone = maxZoneIndex; zone >= 0; zone--) 
   {
     SPI.transfer(bitmap[row][zone]);
   }

   PORTB &= ~(1<<PB2); // set pin 10 (SS) low
   SPI.endTransaction();
   delayMicroseconds(300); // multiples of 3-4 works best
 }
}

void probeSerial() // checks to see if there is data in the serial buffer

{
   if(Serial.available())
   {       
        for(i=0; i<99; i++)
        {
            message[i] = '\0';
        } 
        //resets the index        
        index=0;
    }

    //while reading the message 
    while(Serial.available() > 0)
    {
       //the message can have up to 100 characters 
      
       if(index < (max_char-1)) 
       {         
           r_char = Serial.read();      // Reads a character
           message[index] = r_char;     // Stores the character in message array
           index++;                     // Increment position
           Serial.println("");
           Serial.print(message); 
           EEPROM_writeAnything(0, message);               
       } 
       
     if(Serial.available()<1)
     {
       Serial.println("");
       Serial.print(F("Message Transfer Complete"));
       Serial.println("");
       Serial.print(F("Please reset to display new message"));
     }
     
    }  
}

// Converts row and colum to bitmap bit and turn it off/on
void Plot(int col, int row, bool isOn)
{
 int zone = col / 8;
 int colBitIndex = x % 8;
 byte colBit = 1 << colBitIndex;
 if (isOn)
   bitmap[row][zone] =  bitmap[y][zone] | colBit;
 else
   bitmap[row][zone] =  bitmap[y][zone] & (~colBit);
}
// Plot each character of the message one column at a time, updated the display, shift bitmap left.
void XProcess()
{
 for (int charIndex=0; charIndex < (sizeof(msg)-1); charIndex++)
 {
   int alphabetIndex = msg[charIndex] - ' ';
   if (alphabetIndex < 0) alphabetIndex=0;
   
   //Draw one character of the message
   // Each character is 5 columns wide, loop two more times to create 2 pixel space betwen characters 
   for (int col = 0; col < 6; col++)
   {
     for (int row = 0; row < 8; row++)
     {
       // Set the pixel to what the alphabet say for columns 0 thru 4, but always leave columns 5 and 6 blank.
       bool isOn = 0; 
       if (col<5) isOn = bitRead( alphabets[alphabetIndex][col], 7-row ) == 1;
       Plot( numCols-1, row, isOn); //Draw on the rightmost column, the shift loop below will scroll it leftward.
     }
     for (int refreshCount=0; refreshCount < scrollspeed; refreshCount++)
       RefreshDisplay();
     //Shift the bitmap one column to left
     for (int row=0; row<8; row++)
     {
       for (int zone=0; zone < numZones; zone++)
       {
         //This right shift would show as a left scroll on display because leftmost column is represented by least significant bit of the byte.
         bitmap[row][zone] = bitmap[row][zone] >> 1;
         // Shift over lowest bit from the next zone as highest bit of this zone.
         if (zone < maxZoneIndex) bitWrite(bitmap[row][zone], 7, bitRead(bitmap[row][zone+1],0));
       }
     }
   }
 }
}
void loop() 
{
 XProcess();
 probeSerial();
}
